from pydantic import BaseModel, RootModel


class MixinFromJson:

    @classmethod
    def from_json(cls, json_str):
        """

        Error-tolerant deserialization

        """
        from fmtr.tools import json_fix
        data = json_fix.from_json(json_str, default={})

        if type(data) is dict:
            self = cls(**data)
        else:
            self = cls(data)

        return self


class Base(BaseModel, MixinFromJson):
    """

    Base model

    """
    ...


class Root(RootModel, MixinFromJson):
    """

    Root (list) model

    """
    ...



import pathlib

import pytest

from fmtr.tools import path_tools
from fmtr.tools.tests.helpers import SERIALIZATION_DATA


@pytest.mark.parametrize(
    'args',
    [
        ['/', 'opt', 'data'],
        ['dir', 'data'],
    ]
)
def test_path_args(args):
    expected = str(pathlib.Path(*args))
    actual = str(path_tools.Path(*args))
    assert actual == expected


@pytest.mark.parametrize(
    'raw, expected',
    [
        (r'C:\test', True),
        (r'd:\test', True),
        (r'u:', True),
        (r'x:\test\file.exe', True),
        (r'\\wsl.localhost\shell\bin', True),
        (r'/opt/data', False),
        (r'/bin/usr/python', False),
        (r'test/path', False),
        (r'test\path', False),
    ]
)
def test_path_is_abs_win_path(raw, expected):
    actual = path_tools.Path.is_abs_win_path(raw)
    assert actual == expected
    actual = path_tools.Path.is_abs_win_path(path_tools.Path(raw, convert_wsl=False))
    assert actual == expected


def test_path_module():
    """



    """
    expected = path_tools.Path(__file__).absolute()
    actual = path_tools.Path.module()
    assert actual == expected


def test_path_package():
    """



    """
    expected = path_tools.Path(__file__).absolute().parent
    actual = path_tools.Path.package()
    assert actual == expected


def test_serialization_json():
    """

    Test round trip for JSON.

    """
    expected = SERIALIZATION_DATA
    path = path_tools.Path.temp() / 'serialization_test.json'
    path.write_json(expected)
    actual = path.read_json()
    path.unlink()
    assert actual == expected


def test_serialization_yaml():
    """

    Test round trip for YAML, but also add some more complex types as they are supported.

    """
    expected = SERIALIZATION_DATA | {
        'path': path_tools.Path('/usr/bin/a/b/c/d.ini'),
        'bytes': b'test',
        'set': {'foo', 'bar', 'baz'},
        'text': 'Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium doloremque laudantium \n' * 100
    }
    path = path_tools.Path.temp() / 'serialization_test.yaml'
    path.write_yaml(expected)
    actual = path.read_yaml()
    path.unlink()
    assert actual == expected



import pytest

from fmtr.tools import datatype_tools
from fmtr.tools.datatype_tools import TypeConversionFailed
from fmtr.tools.tests import helpers
from fmtr.tools.tools import Raise


@helpers.parametrize(
    'raw, expected',
    [
        ('true', True),
        ('1', True),
        ('t', True),
        ('y', True),
        ('yes', True),
        ('false', False),
        (0, False),
        ('f', False),
        ('n', False),
        ('no', False),
        ('nooo', None),
        ('zzz', None),
        ('zzz', Raise),
    ]
)
def test_to_bool(raw, expected):
    if expected is Raise:
        with pytest.raises(TypeConversionFailed):
            datatype_tools.to_bool(raw, default=Raise)
    else:
        actual = datatype_tools.to_bool(raw)
        assert actual == expected
